/**
 * @author stephan
 * based upon the Tooltip class in Javascript: The Definitive Guide 5th Edition by David Flanagan
 * uses classes
 * the correct css classes must be in the stylesheets for this tooltip to display correctly
 * eg.
 * .tooltipShadow {background: url(shadow.png)};
 * .tooltipContent {
 * 		left: -4px; top: -4px;      //amount of shadow to display on which sides 
 * 		background-color: #ff9494;  //ruby background color
 * 		border: solid black 1px;
 * 		padding: 5px;
 * 		font: bold 10pt Arial; 
 * 	};
 */

Tooltip.tooltip = new Tooltip("jsHint");

var RelativeToTarget = true;  // set to true if want to use target for example TD as reference instead of mouse


//Tooltip constructor
function Tooltip(id) {
	this.tooltip = document.createElement("DIV"); //div for shadow	
	this.tooltip.style.position = "absolute";     //needs to be absolutely positioned
	this.tooltip.style.visibility = "hidden";     //start of hidden
	this.tooltip.className        = "tooltipShadow";
	this.tooltip.width            = "256px";
	this.tooltip.height           = "192px";	
	
	this.content = document.createElement("DIV"); //create div for content
	this.content.style.position = "relative"; 		//positioned relatively to the shadow
	this.content.className = "tooltipContent";
	
	this.tooltip.appendChild(this.content);				//add content to the shadow
};


function CalcElementRect(oStartElement)	{ 
			var lOffsetTop, lOffsetLeft; 
			var oCurrElement; 
			lOffsetTop = 0; 
			lOffsetLeft = 0; 
			for(oCurrElement = oStartElement; (oCurrElement != null) && (oCurrElement.tagName != 'BODY'); oCurrElement = oCurrElement.offsetParent) 
			{	lOffsetTop  += oCurrElement.offsetTop; 
				lOffsetLeft += oCurrElement.offsetLeft; 
			} 
			return { Top: lOffsetTop, Left: lOffsetLeft, Bottom: lOffsetTop + oStartElement.offsetHeight - 1, Right: lOffsetLeft + oStartElement.offsetWidth - 1 }; 
}

//show function prototype

Tooltip.prototype.show = function(target, text,mouseX, mouseY) {

	//convert mouse coordinates to document coordinates using geometry module
	//add the offest so the tooltip doesn't appear right under the mouse

/*	text = 'Amethyst provides three methods of recording. '
			+  'Method 1 is client side where we record via a phone tap through a PC to a central server.'
			+  ' This method provides the most cost effective way to record, the most features of all the methods, '
			+  'as well as full client side integration options. Method 2 is extension side recording. '
			+  'This method provides a more robust method of recording, but still with all the features of '
			+  'client side recording. This method works extremely well in Citrix or thin client environments. '
			+  'Method 3 is trunk side recording. This is an extremely powerful way of recording with absolutely '
			+  'minimum hardware in place. Although lacking some of the features of client and extension side recording, '
			+  'this method can be very cost effective if individual agent monitoring or AQM is not a pre-requisite. '
			+  'Amethyst allows the end user the opportunity to maximize the recording solution by allowing the end user a '
			+  'mixture of any of the recording methods listed here. Please ask your Amethyst Sales Partner for more detailed information.'; */ 
   text = text.replace(/#!/g,"'")
   text = text.replace(/#~/g,'"')   
	 this.content.innerHTML =  text;

	 //add the tool tip to the document if it has not been added before
	 if (this.tooltip.parentNode != document.body)
	 	document.body.appendChild(this.tooltip);

	////Winston Tuck
   var divWidth = this.tooltip.offsetWidth;	
   var divHeight = this.tooltip.offsetHeight;	
   var x = 0;
   var y = 0;

   // calculate the x and Y under normal circumstances
	 if (RelativeToTarget==true) {    //RelativeToTarget if using element instead of mouse as positioner.
		 var oTargetRect = CalcElementRect(target);
		 y = oTargetRect.Bottom + Tooltip.Y_OFFSET;
		 x = oTargetRect.Left + Tooltip.X_OFFSET; 		 
	 }	else {
     x = mouseX + Geometry.getHorizontalScroll() + Tooltip.X_OFFSET;
     y = mouseY + Geometry.getVerticalScroll() + Tooltip.Y_OFFSET;	 
	 }

   //check if not running of end of screen	 
   if ((y + divHeight) > (Geometry.getViewportHeight()+Geometry.getVerticalScroll())) 
   { if (RelativeToTarget==true) { y = oTargetRect.Top}
   	 else {y = mouseY + Geometry.getVerticalScroll()};
   	 
   	 
   	 y = y - Tooltip.Y_OFFSET - divHeight;
   }

   if ((x + divWidth) > (Geometry.getViewportWidth()+Geometry.getHorizontalScroll())) 
   {x = mouseX + Geometry.getHorizontalScroll() - Tooltip.X_OFFSET - divWidth};   	 


//   if ((y + divHeight) > (Geometry.getViewportHeight()+Geometry.getVerticalScroll())) {y = mouseY + Geometry.getVerticalScroll() - Tooltip.Y_OFFSET - divHeight};
//   if ((x + divWidth) > (Geometry.getViewportWidth()+Geometry.getHorizontalScroll())) {x = mouseX + Geometry.getHorizontalScroll() - Tooltip.X_OFFSET - divWidth};   

	 // compensate for running at negative starts 
   if (x < Geometry.getHorizontalScroll()) {x = Geometry.getHorizontalScroll() + 20};
   if (y < Geometry.getVerticalScroll()) {y = Geometry.getVerticalScroll() + 20};
   
   this.tooltip.style.left = x + "px";
	 this.tooltip.style.top  = y + "px";  

   this.tooltip.style.visibility = "visible";   
	 	
	 //check for command once... like with the live_hint updates
	 var commandOnceInput = document.getElementById("commandOnce");
	 if (commandOnceInput) {
		 var commandOnce = commandOnceInput.value;
		 var sessionIdInput = document.getElementById("sessionIdOnce");
		 var sessionId = sessionIdInput.value;
		 try {		
		   doRequest(commandOnce,sessionId);
		 } catch (e) {
		 };
	  };
};

Tooltip.prototype.hide = function() {
	this.tooltip.style.visibility = "hidden";
};





//these values are used by the schedule method
//they are used as constants, but can be set if the developer wants different offsets
Tooltip.X_OFFSET = 25; //pixels to the right of the mouse pointer
Tooltip.Y_OFFSET = 15; //pixels below the mouse pointer
Tooltip.DELAY    = 500; //milliseconds after mouseover

Tooltip.prototype.schedule = function(target,e,text) {
	//event holds mouse position coordinates

	var mouseX = e.clientX; //Winston Tuck
	var mouseY = e.clientY; //Winston Tuck
	
	//schedule the display of the tooltip
	var self = this;  //we need this for nested functions
	var timer = window.setTimeout(function() {self.show(target, text,mouseX, mouseY);},Tooltip.DELAY);

	//also register an onmouseout to hide a tooltip or cancel the pending display
	  if (target.addEventListener) 
		target.addEventListener("mouseout",mouseout,false);
	else if (target.attachEvent)
		target.attachEvent("onmouseout",mouseout);
	else
		target.onmouseout = mouseout;  
		
	//implementation of the event listener
	function mouseout() {
		self.hide();
		window.clearTimeout(timer);
		//remove the event handler
		if (target.removeEventListener) 
			{target.removeEventListener("mouseout",mouseout,false)}
		else if (target.detachEvent)
			{target.detachEvent("onmouseout",mouseout)}
		else
			{target.onmouseout = nil}
	};
	
};


Tooltip.schedule = function(target,e,text) {Tooltip.tooltip.schedule(target,e,text);};